"""__init__"""
from .analyser import Analyser
from .class_instance import ClassInstanceAnalyser
from .encapsulation import EncapsulationAnalyser
from .naming import NamingAnalyser
from .scope import ScopeAnalyser
from .structure import StructureAnalyser
